##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##
##----------  Step 0: Change the following to your working directory -------------##
##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##

setwd("/Users/...")

##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##
##----------      Step 1: Prepare data & load related packages     ---------------##
##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##

rm(list=ls())
penalty <- seq(from = 0, to = 2, by = 0.05)
l.penalty <- length(penalty)

library(glmnet)
rf <- read.csv("rf.csv",header=T)


rawdata1 <- read.csv("100_Portfolios_ME_INV.csv",header=T)
rawdata2 <- read.csv("100_Portfolios_ME_OP.csv",header=T)
rawdata3 <- read.csv("100_Portfolios.csv",header=T)
rawdata <- cbind(rawdata1,rawdata2[,-1],rawdata3[,-1])



##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##
##----------   Step 2 : Create an empty matrix containing        -----------------##
#-------------         all dates we want for the exercise        -----------------##
##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##


rawdata$date <- as.numeric(substr(rawdata$date,1,6))
rf$date<- as.numeric(substr(rf$date,1,6))

MonthsVector <- c("01","02","03","04","05","06","07","08","09","10","11","12")

startyear <- as.numeric(substr(199307,1,4))
startmonth <- substr(199307,5,6)
endyear <- as.numeric(substr(201012,1,4))
endmonth <- substr(201012,5,6)
nbofyears <- endyear - startyear
matrixofdates <- NULL
for(i in 0:nbofyears){
  for(r in 1:12){
    matrixofdates <- rbind(matrixofdates,c(startyear+i,MonthsVector[r]))
  }
}

matrixofdates <- matrixofdates[7:nrow(matrixofdates),]

newmatrix <- matrix(NA, nrow = nrow(matrixofdates), ncol = l.penalty)


matrixofdates <- cbind(matrixofdates,newmatrix)


colnames(matrixofdates)[3:ncol(matrixofdates)] <- penalty


matrixofsparsity <- matrixofdates


##--------------------------------------------------------------------------------##
##----------  Step 3: create a loop to extract out-of-sample SDF  ----------------##
#-----------            in a rolling fashion                      ----------------##
##--------------------------------------------------------------------------------##

for(i in 1:nrow(matrixofdates)) {
  
    if (matrixofdates[i,2] %in% c("07") & i == 205){
   
      predictiondate<-as.numeric(paste(matrixofdates[i,1], matrixofdates[i,2],sep=""))
      predictionenddate <- as.numeric(paste(matrixofdates[i+5,1], matrixofdates[i+5,2],sep=""))  
      
      predictionyear<-as.numeric(substr(predictiondate,1,4))
      predictionmonth<-substr(predictiondate,5,6)
       lastmonth <- MonthsVector[as.numeric(predictionmonth)-1]
       lastyear<- predictionyear
       beginyear<- lastyear-30
       beginmonth<-predictionmonth
       begindate <- as.numeric(paste(beginyear, beginmonth,sep=""))
       lastdate <- as.numeric(paste(lastyear,lastmonth,sep=""))
       

       toy <- rawdata[which(rawdata$date >=begindate & rawdata$date <=predictionenddate),]
       toyrf <- rf[which(rf$date  >=begindate & rf$date <= predictionenddate),]
       toy <- toy[!colSums(toy)%in% NA]
       
       asset.no<- ncol(toy)-1
       
       totalx <- toy[,-1]
       toyrf <-toyrf[,2]
       totalrfm <- matrix(toyrf, nrow=length(toyrf),ncol=asset.no)
       toyrfm <- totalrfm
       
       #calculate excess return#
       totalx  <- totalx-toyrfm
       
       x<- as.matrix(totalx[1:(nrow(totalx)-6),])
       
       xpredict <- as.matrix(tail(totalx, 6))
       
       y<- rep(0,360)
       y <- as.matrix(y)
       
       fit = glmnet(x, y, family = "poisson", intercept=FALSE, standardize=FALSE,alpha=1)
       
       predict1 <- predict(fit, newx=xpredict, s=penalty, type="response")
       
       dqdp <- sweep(predict1, 2, colMeans(predict1), "/" )
       mhat <- dqdp
      
       matrixofdates[i:(i+5),3:ncol(matrixofdates)] <- mhat
       
       
    } else if (matrixofdates[i,2] %in% c("07") & i!= 205){  

      predictiondate<-as.numeric(paste(matrixofdates[i,1], matrixofdates[i,2],sep=""))
      predictionenddate <- as.numeric(paste(matrixofdates[i+11,1], matrixofdates[i+11,2],sep=""))  
      
      predictionyear<-as.numeric(substr(predictiondate,1,4))
      predictionmonth<-substr(predictiondate,5,6)
      lastmonth <- MonthsVector[as.numeric(predictionmonth)-1]
      lastyear<- predictionyear
      beginyear<- lastyear-30
      beginmonth<-predictionmonth
      begindate <- as.numeric(paste(beginyear, beginmonth,sep=""))
      lastdate <- as.numeric(paste(lastyear,lastmonth,sep=""))
      
      
      toy <- rawdata[which(rawdata$date >=begindate & rawdata$date <=predictionenddate),]
      toyrf <- rf[which(rf$date  >=begindate & rf$date <= predictionenddate),]
      
      toy <- toy[!colSums(toy)%in% NA]
      
      asset.no<- ncol(toy)-1
      
      totalx <- toy[,-1]

      toyrf <-toyrf[,2]

      totalrfm <- matrix(toyrf, nrow=length(toyrf),ncol=asset.no)
      
      toyrfm <- totalrfm
  
      totalx  <- totalx-toyrfm
      
      x<- as.matrix(totalx[1:(nrow(totalx)-12),])

      xpredict <- as.matrix(tail(totalx, 12))
      

      y<- rep(0,360)
      y <- as.matrix(y)
      
      fit = glmnet(x, y, family = "poisson", intercept=FALSE, standardize=FALSE,alpha=1)
      

      predict1 <- predict(fit, newx=xpredict, s=penalty, type="response")
      
      dqdp <- sweep(predict1, 2, colMeans(predict1), "/" )  
      
      mhat <- dqdp
   
    matrixofdates[i:(i+11),3:ncol(matrixofdates)] <- mhat
    
  }
       
  print(paste(round(i/nrow(matrixofdates)*100),"% done for the loop.",sep=""))
  

}

##--------------------------------------------------------------------------------##
##----------  Step 3-1: save  out-of-sample SDF to a csv file      ---------------##
##--------------------------------------------------------------------------------##
write.csv(matrixofdates,file="300ff.KL.sdf.csv")

##--------------------------------------------------------------------------------##
##----------  Step 4: Conduct cross sectional regression tests     ---------------##
#-----------            using extracted SDF                       ----------------##
##--------------------------------------------------------------------------------##


pdata <- read.csv("300_Portfolios_fortest.csv", header=T)

pdata <- pdata[1:210,]
pdata <- pdata[!colSums(pdata)%in% NA]

t <- (ncol(pdata)-1)

table3.SDF.KL <- matrix(NA,ncol = 5,nrow = l.penalty)

colnames(table3.SDF.KL ) <-c("con","lamda","t.con","t.lamda", "r2")
rownames(table3.SDF.KL) <- penalty

for (j in 3:(2+l.penalty)){
  
  #pick our recovered sdf
  sdf <- as.matrix(as.numeric(matrixofdates[,j]))
  
  #create an empty vector of length 25, which is the number of testing assets
  v.betas <- matrix(NA,  nrow = t, ncol = 1)
  
  #for each asset, run time series regresstion to get beta
  for (i in 1:t){
    
    y<- pdata[ ,(i+1)]
    fit1 <- lm( y ~ sdf)
    
    v.betas[i,1] <-coefficients(fit1)[2]
    
  }
  
  
  #calculate mean return for each asset
  pdata1 <- pdata[,2:ncol(pdata)]
  meanr <- as.vector(colMeans(pdata1))
  
  #second pass, run mean return on estimated beta
  finalfit <- lm(meanr ~ v.betas)
  output<-summary(finalfit)
  
  table3.SDF.KL[(j-2),1:2] <- output$coefficients[,1]
  table3.SDF.KL[(j-2),3:4] <- output$coefficients[,3]
  table3.SDF.KL[(j-2),5] <- output$adj.r.squared
  
}

##--------------------------------------------------------------------------------##
##----------  Step 4-1: save  cross sectional regression results  to a csv file --##
##--------------------------------------------------------------------------------##
write.csv(table3.SDF.KL,file="table3.SDF.KL.csv")
