##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##
##----------  Step 0: Change the following to your working directory -------------##
##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##

setwd("/Users/...")

##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##
##----------      Step 1: Prepare data & load related packages     ---------------##
##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##

rm(list=ls())
penalty <- seq(from = 0.55, to = 1, by = 0.05)

l.penalty <- length(penalty)

library(lbfgs)
rf <- read.csv("rf.csv",header=T)

rawdata1 <- read.csv("100_Portfolios_ME_INV.csv",header=T)
rawdata2 <- read.csv("100_Portfolios_ME_OP.csv",header=T)
rawdata3 <- read.csv("100_Portfolios.csv",header=T)
rawdata <- cbind(rawdata1,rawdata2[,-1],rawdata3[,-1])



##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##
##----------   Step 2 : Create an empty matrix containing        -----------------##
#-------------         all dates we want for the exercise        -----------------##
##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##

rawdata$date <- as.numeric(substr(rawdata$date,1,6))
rf$date<- as.numeric(substr(rf$date,1,6))

MonthsVector <- c("01","02","03","04","05","06","07","08","09","10","11","12")

startyear <- as.numeric(substr(199307,1,4))
startmonth <- substr(199307,5,6)
endyear <- as.numeric(substr(201012,1,4))
endmonth <- substr(201012,5,6)
nbofyears <- endyear - startyear
matrixofdates <- NULL
for(i in 0:nbofyears){
  for(r in 1:12){
    matrixofdates <- rbind(matrixofdates,c(startyear+i,MonthsVector[r]))
  }
}

matrixofdates <- matrixofdates[7:nrow(matrixofdates),]


newmatrix <- matrix(NA, nrow = nrow(matrixofdates), ncol = l.penalty)


matrixofdates <- cbind(matrixofdates,newmatrix)


colnames(matrixofdates)[3:ncol(matrixofdates)] <- penalty

matrixofsparsity <- matrixofdates


##--------------------------------------------------------------------------------##
##----------  Step 3: create a loop to extract out-of-sample SDF  ----------------##
#-----------            in a rolling fashion                      ----------------##
##--------------------------------------------------------------------------------##


for(i in 1:nrow(matrixofdates)) {
  
    if (matrixofdates[i,2] %in% c("07") & i == 205){
   
      predictiondate<-as.numeric(paste(matrixofdates[i,1], matrixofdates[i,2],sep=""))
      predictionenddate <- as.numeric(paste(matrixofdates[i+5,1], matrixofdates[i+5,2],sep=""))  
    
      
      predictionyear<-as.numeric(substr(predictiondate,1,4))
      predictionmonth<-substr(predictiondate,5,6)
       lastmonth <- MonthsVector[as.numeric(predictionmonth)-1]
       lastyear<- predictionyear
       beginyear<- lastyear-30
       beginmonth<-predictionmonth
       begindate <- as.numeric(paste(beginyear, beginmonth,sep=""))
       lastdate <- as.numeric(paste(lastyear,lastmonth,sep=""))
       

       toy <- rawdata[which(rawdata$date >=begindate & rawdata$date <=predictionenddate),]
       toyrf <- rf[which(rf$date  >=begindate & rf$date <= predictionenddate),]
    
       toy <- toy[!colSums(toy)%in% NA]
       
       asset.no<- ncol(toy)-1
       
       totalx <- toy[,-1]

       toyrf <-toyrf[,2]
   
       totalrfm <- matrix(toyrf, nrow=length(toyrf),ncol=asset.no)
       
 
       toyrfm <- totalrfm
  
       totalx  <- cbind(rep(1,nrow(totalx)),(totalx-toyrfm)) 
       
 
       x<- as.matrix(totalx[1:(nrow(totalx)-6),])
   
       xpredict <- as.matrix(tail(totalx, 6))
       

       #optimization: PSN1 #
       
       
       objective <- function(par){
         ob1 <- (sum((x %*% par)*(x %*% par))/nrow(x))/2 + sum(x %*% par)/nrow(x)
         ob2 <- sum(par*c(1,rep(0,ncol(x)-1)))
         
         return(ob1-ob2)
       }
       
       gradient <- function(par){
         gr1 <- (crossprod(x)/nrow(x)) %*% par + t(crossprod(rep(1,nrow(x)),x)/nrow(x))
         gr2 <- c(1,rep(0,ncol(x)-1))
         
         return(gr1-gr2)
       }
       
       init <- rep(0,ncol(x))
       
       psn.sdf <- function(pen){
         
         output <-lbfgs(objective, gradient, init, invisible=1, orthantwise_c=pen,orthantwise_start = 2,orthantwise_end =ncol(x))
        
         est.coeff <- output$par
         
         
         predict1 <- xpredict %*% est.coeff+1
         mhat <- predict1
                        
         
         return(mhat)
         
       }

       matrixofdates[i:(i+5),3:(length(penalty)+2)] <-sapply(penalty,psn.sdf)
 
       
    } else if (matrixofdates[i,2] %in% c("07") & i!= 205){  

      predictiondate<-as.numeric(paste(matrixofdates[i,1], matrixofdates[i,2],sep=""))
      predictionenddate <- as.numeric(paste(matrixofdates[i+11,1], matrixofdates[i+11,2],sep=""))  
      
      
      predictionyear<-as.numeric(substr(predictiondate,1,4))
      predictionmonth<-substr(predictiondate,5,6)
      lastmonth <- MonthsVector[as.numeric(predictionmonth)-1]
      lastyear<- predictionyear
      beginyear<- lastyear-30
      beginmonth<-predictionmonth
      begindate <- as.numeric(paste(beginyear, beginmonth,sep=""))
      lastdate <- as.numeric(paste(lastyear,lastmonth,sep=""))
      
      
      
      toy <- rawdata[which(rawdata$date >=begindate & rawdata$date <=predictionenddate),]
      toyrf <- rf[which(rf$date  >=begindate & rf$date <= predictionenddate),]
      
      toy <- toy[!colSums(toy)%in% NA]
      

      asset.no<- ncol(toy)-1
      
      totalx <- toy[,-1]
 
      toyrf <-toyrf[,2]

      totalrfm <- matrix(toyrf, nrow=length(toyrf),ncol=asset.no)
      
  
      toyrfm <- totalrfm
      
      totalx  <- cbind(rep(1,nrow(totalx)),(totalx-toyrfm)) 
      
      x<- as.matrix(totalx[1:(nrow(totalx)-12),])

      xpredict <- as.matrix(tail(totalx, 12))
      
      #optimization: PSN1 #
      
      objective <- function(par){
        ob1 <- (sum((x %*% par)*(x %*% par))/nrow(x))/2 + sum(x %*% par)/nrow(x)
        ob2 <- sum(par*c(1,rep(0,ncol(x)-1)))
        
        return(ob1-ob2)
      }
      
      gradient <- function(par){
        gr1 <- (crossprod(x)/nrow(x)) %*% par + t(crossprod(rep(1,nrow(x)),x)/nrow(x))
        gr2 <- c(1,rep(0,ncol(x)-1))
        
        return(gr1-gr2)
      }
      
      init <- rep(0,ncol(x))
      
      objective <- function(par){
        ob1 <- (sum((x %*% par)*(x %*% par))/nrow(x))/2 + sum(x %*% par)/nrow(x)
        ob2 <- sum(par*c(1,rep(0,ncol(x)-1)))
        
        return(ob1-ob2)
      }
      
      gradient <- function(par){
        gr1 <- (crossprod(x)/nrow(x)) %*% par + t(crossprod(rep(1,nrow(x)),x)/nrow(x))
        gr2 <- c(1,rep(0,ncol(x)-1))
        
        return(gr1-gr2)
      }
      
      init <- rep(0,ncol(x))
      
      psn.sdf <- function(pen){
        
        output <-lbfgs(objective, gradient, init, invisible=1, orthantwise_c=pen,orthantwise_start = 2,orthantwise_end =ncol(x))
        
        est.coeff <- output$par
        
        
        predict1 <- xpredict %*% est.coeff+1
        mhat <- predict1
        
        
        return(mhat)
        
      }
      
    
    matrixofdates[i:(i+11),3:(length(penalty)+2)] <- sapply(penalty,psn.sdf)
      
 
  }
       
  print(paste(round(i/nrow(matrixofdates)*100),"% done for the loop.",sep=""))
  

}

##--------------------------------------------------------------------------------##
##----------  Step 3-1: save  out-of-sample SDF to a csv file      ---------------##
##--------------------------------------------------------------------------------##
write.csv(matrixofdates,file="300ff.PSN.sdf.2.csv")

