##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##
##----------  Step 0: Change the following to your working directory -------------##
##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##

setwd("/Users/...")

##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##
##----------      Step 1: Prepare data & load related packages     ---------------##
##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##

rm(list=ls())
library(moments)

KL.SDF.high<- read.csv("300ff.KL.sdf.csv", header=T)
PSN.SDF.high<-read.csv("300ff.PSN.sdf.3.csv", header=T)

##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##
##----------           Step 2: Replicate Table 4                   ---------------##
##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##

colnames(KL.SDF.high) [2] <- "Year"
colnames(KL.SDF.high) [3] <-"Month"


KL.optimal <-KL.SDF.high[,"X0.9"]
PSN.optimal <- PSN.SDF.high[,"X0.475"]


fffactor <- as.matrix(read.csv("F-F__Factors.CSV", header=T))

momfactor <- read.csv("F-F_Momentum_Factor.CSV", header=T)
momfactor <- as.matrix((momfactor[1:210,]))

Table4.A <- matrix(NA, ncol=6, nrow=2)
Table4.B <- matrix(NA, ncol=6, nrow=2)

fitA <- lm(KL.optimal ~ fffactor[,2:4]+as.numeric(momfactor[,2]))
output.A <-summary(fitA)
Table4.A[1,1:5] <- output.A$coefficients[,1]
Table4.A[2,1:5] <- output.A$coefficients[,3]
Table4.A[1,6] <- output.A$adj.r.squared



fitB <- lm(PSN.optimal ~ fffactor[,2:4]+as.numeric(momfactor[,2]))
output.B <-summary(fitB)
Table4.B[1,1:5] <- output.B$coefficients[,1]
Table4.B[2,1:5] <- output.B$coefficients[,3]
Table4.B[1,6] <- output.B$adj.r.squared


write.csv(Table4.A,file="Table4.A.csv")
write.csv(Table4.B,file="Table4.B.csv")


##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##
##----------           Step 2: Replicate Figure 2                  ---------------##
##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##

dev.off()
date <- with(KL.SDF.high,as.Date(paste(Year,Month,'1',sep='-'),format='%Y-%m-%d'))


plot(PSN.optimal~date,type='n',
     xaxt='n',ylab="Predicted SDF", xlab="",main="",col="lightskyblue4",lwd=1.5)
rect(c(date[93],date[174]),
     -1e6, c(date[101],date[192]), 1e6, col='lightgrey', border=NA)

lines(KL.optimal~date,lty=1,col="black",lwd=1.5)
lines(PSN.optimal~date,lty=2,col="#636363",lwd=1.5)

xmin<-min(date,na.rm=T);xmax<-max(date,na.rm=T)
xseq<-seq.Date(xmin,xmax,by='12 month') 
axis.Date(side=1,at=xseq,format='%Y-%m',labels=T,las=2)
title(xlab="Date", line=4)
legend("topleft",c(expression(paste("KL, ",alpha[n],"=0.9")),expression(paste("PSN1, ",alpha[n],"=0.475"))),lty=c(1,2), col=c("black","#636363"),
       lwd=c(1.5,1.5), cex = 0.4)


