##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##
##----------  Step 0: Change the following to your working directory -------------##
##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##

setwd("/Users/...")

##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##
##----------      Step 1: Prepare data & load related packages     ---------------##
##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##


rm(list=ls())
library(glmnet)
rf <- read.csv("rf.csv",header=T)
rawdata <- read.csv("25FFm.csv",header=T)


##----------------------------------------##
##----------------Trimming----------------##
##----------------------------------------##

#from 1963-07 to 2020-06#
traindata <- rawdata[(85:1008),-1]
trainrf   <- rf[(85:1008),-1]
traindata <- traindata 
trainrf   <- trainrf
trainrfm <- matrix(trainrf, nrow=length(trainrf),ncol=25)
trainex <-abs(traindata -trainrfm)

supex <- apply(trainex, 1, max)

#first trimming boundary#
q1 <-   quantile(supex,0.99)
#second trimming boundary#
q2 <- quantile(supex,0.975)


##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##
##----------   Step 2 : Create an empty matrix containing        -----------------##
#-------------         all dates we want for the exercise        -----------------##
##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##




rawdata$date <- as.numeric(substr(rawdata$date,1,6))
rf$date<- as.numeric(substr(rf$date,1,6))

MonthsVector <- c("01","02","03","04","05","06","07","08","09","10","11","12")

startyear <- as.numeric(substr(196307,1,4))
startmonth <- substr(196307,5,6)
endyear <- as.numeric(substr(201012,1,4))
endmonth <- substr(201012,5,6)
nbofyears <- endyear - startyear
matrixofdates <- NULL
for(i in 0:nbofyears){
  for(r in 1:12){
    matrixofdates <- rbind(matrixofdates,c(startyear+i,MonthsVector[r]))
  }
}

matrixofdates <- matrixofdates[7:nrow(matrixofdates),]


newmatrix <- matrix(, nrow = nrow(matrixofdates), ncol = 2)

matrixofdates <- cbind(matrixofdates,newmatrix)


colnames(matrixofdates)[3:4] <- c("trim1","trim2")


##--------------------------------------------------------------------------------##
##----------  Step 3: create a loop to extract out-of-sample SDF  ----------------##
#-----------            in a rolling fashion                      ----------------##
##--------------------------------------------------------------------------------##


for(i in 1:nrow(matrixofdates)) {
  

    if (matrixofdates[i,2] %in% c("07") & i == 565){
      
      predictiondate<-as.numeric(paste(matrixofdates[i,1], matrixofdates[i,2],sep=""))
      predictionenddate <- as.numeric(paste(matrixofdates[i+5,1], matrixofdates[i+5,2],sep=""))  
    
      predictionyear<-as.numeric(substr(predictiondate,1,4))
      predictionmonth<-substr(predictiondate,5,6)
       lastmonth <- MonthsVector[as.numeric(predictionmonth)-1]
       lastyear<- predictionyear
       beginyear<- lastyear-30
       beginmonth<-predictionmonth
       begindate <- as.numeric(paste(beginyear, beginmonth,sep=""))
       lastdate <- as.numeric(paste(lastyear,lastmonth,sep=""))
       

       toy <- rawdata[which(rawdata$date >=begindate & rawdata$date <=predictionenddate),]
       toyrf <- rf[which(rf$date  >=begindate & rf$date <= predictionenddate),]
    

       totalx <- toy[,-1]
       
       #extract risk free return
       toyrf <-toyrf[,2]

       totalrfm <- matrix(toyrf, nrow=length(toyrf),ncol=25)
 
       toyrfm <- totalrfm
 
       totalx  <- totalx-toyrfm
       
       x<- as.matrix(totalx[1:(nrow(totalx)-6),])
       
       
       ##apply trimming#
       supx <- apply(abs(x), 1, max)

       
       #first data uses q1 boundary#
       x1 <- x[(supx <= q1),]
       #second data uses q2 boundary#
       x2 <- x[(supx <= q2),]
       
       
       #creation of prediction matrix
       xpredict <- as.matrix(tail(totalx, 6))
       
 
    
       y1 <- as.matrix(rep(0,nrow(x1)))
       y2 <- as.matrix(rep(0,nrow(x2)))
 

       fit1 = glmnet(x1, y1, family = "poisson", intercept=FALSE, standardize=FALSE,alpha=1)
       fit2 = glmnet(x2, y2, family = "poisson", intercept=FALSE, standardize=FALSE,alpha=1)

       predict1 <- cbind(predict(fit1, newx=xpredict, s=0, type="response"),
                  predict(fit2, newx=xpredict, s=0, type="response"))
       

       dqdp <- sweep(predict1, 2, colMeans(predict1), "/" )
       mhat <- dqdp
       
       matrixofdates[i:(i+5),3:4] <- mhat
       
    } else if (matrixofdates[i,2] %in% c("07") & i!= 565){  
   
      predictiondate<-as.numeric(paste(matrixofdates[i,1], matrixofdates[i,2],sep=""))
      predictionenddate <- as.numeric(paste(matrixofdates[i+11,1], matrixofdates[i+11,2],sep=""))  
      
      predictionyear<-as.numeric(substr(predictiondate,1,4))
      predictionmonth<-substr(predictiondate,5,6)
      lastmonth <- MonthsVector[as.numeric(predictionmonth)-1]
      lastyear<- predictionyear
      beginyear<- lastyear-30
      beginmonth<-predictionmonth
      begindate <- as.numeric(paste(beginyear, beginmonth,sep=""))
      lastdate <- as.numeric(paste(lastyear,lastmonth,sep=""))
      
      
      toy <- rawdata[which(rawdata$date >=begindate & rawdata$date <=predictionenddate),]
      toyrf <- rf[which(rf$date  >=begindate & rf$date <= predictionenddate),]
      

      totalx <- toy[,-1]

      #extract risk free return
      toyrf <-toyrf[,2]
      length(toyrf)
      
      totalrfm <- matrix(toyrf, nrow=length(toyrf),ncol=25)

      toyrfm <- totalrfm
      
      totalx  <- totalx-toyrfm
      
      x<- as.matrix(totalx[1:(nrow(totalx)-12),])
      
      ##apply trimming#
      supx <- apply(abs(x), 1, max)

      #first data uses q1 boundary#
      x1 <- x[(supx <= q1),]
      #second data uses q2 boundary#
      x2 <- x[(supx <= q2),]
      
      #creation of prediction matrix
      xpredict <- as.matrix(tail(totalx, 12))
      
  
      y1 <- as.matrix(rep(0,nrow(x1)))
      y2 <- as.matrix(rep(0,nrow(x2)))
      
      fit1 = glmnet(x1, y1, family = "poisson", intercept=FALSE, standardize=FALSE,alpha=1)
      fit2 = glmnet(x2, y2, family = "poisson", intercept=FALSE, standardize=FALSE,alpha=1)
      
      
      predict1 <- cbind(predict(fit1, newx=xpredict, s=0, type="response"),
                        predict(fit2, newx=xpredict, s=0, type="response"))
      
      dqdp <- sweep(predict1, 2, colMeans(predict1), "/" )
      mhat <- dqdp
    
    matrixofdates[i:(i+11),3:4] <- mhat
    
  }
       
  print(paste(round(i/nrow(matrixofdates)*100),"% done for the loop.",sep=""))
  

}

##--------------------------------------------------------------------------------##
##----------  Step 3-1: save  out-of-sample SDF to a csv file      ---------------##
##--------------------------------------------------------------------------------##
write.csv(matrixofdates,file="25ff.sdf.est.KL.trim.csv")

##--------------------------------------------------------------------------------##
##----------  Step 4: Conduct cross sectional regression tests     ---------------##
#-----------            using extracted SDF                       ----------------##
##--------------------------------------------------------------------------------##

pdata <- read.csv("25_Portfolios_5x5.csv", header=T)
pdata <- pdata[1:570,]

t <- (ncol(pdata)-1)
Table2.B.SDF.KL.t <- matrix(NA,ncol = 5,nrow = 2)

colnames(Table2.B.SDF.KL.t) <-c("con","lamda","t.con","t.lamda", "r2")
rownames(Table2.B.SDF.KL.t) <- c("trim1","trim2")

for (j in 3:4){
  
  
  #pick our recovered sdf
  sdf <- as.matrix(as.numeric(matrixofdates[,j]))
  
  #create an empty vector of length 25, which is the number of testing assets
  v.betas <- matrix(,  nrow = t, ncol = 1)
  
  #for each asset, run time series regression to get beta
  for (i in 1:t){
    
    y<- pdata[,(i+1)]
    fit1 <- lm( y ~ sdf)
    
    v.betas[i,1] <-coefficients(fit1)[2]
    
  }
  
  
  #calculate mean return for each asset
  pdata1 <- pdata[,2:ncol(pdata)]
  meanr <- as.vector(colMeans(pdata1))
  
  #second pass, run mean return on estimated beta
  finalfit <- lm(meanr ~ v.betas)
  output<-summary(finalfit)
  
  Table2.B.SDF.KL.t[(j-2),1:2] <- output$coefficients[,1]
  Table2.B.SDF.KL.t[(j-2),3:4] <- output$coefficients[,3]
  Table2.B.SDF.KL.t[(j-2),5] <- output$adj.r.squared
  
}

##--------------------------------------------------------------------------------##
##----------  Step 4-1: save  cross sectional regression results  to a csv file --##
##--------------------------------------------------------------------------------##
write.csv(Table2.B.SDF.KL.t,file="Table2.B.SDF.KL.trim.csv")