##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##
##----------  Step 0: Change the following to your working directory -------------##
##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##

setwd("/Users/...")

##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##
##----------      Step 1: Prepare data & load related packages     ---------------##
##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##

rm(list=ls())
library(lbfgs)
library(MASS)
rf <- read.csv("rf.csv",header=T)
rawdata <- read.csv("25FFm.csv",header=T)


##----------------------------------------##
##----------------Trimming----------------##
##----------------------------------------##


#from 1963-07 to 2020-06#
traindata <- rawdata[(85:1008),-1]
trainrf   <- rf[(85:1008),-1]
traindata <- traindata
trainrf   <- trainrf
trainrfm <- matrix(trainrf, nrow=length(trainrf),ncol=25)
trainex <-abs(traindata -trainrfm)

supex <- apply(trainex, 1, max)

#first trimming boundary#
q1 <-   quantile(supex,0.99)
#second trimming boundary#
q2 <- quantile(supex,0.975)



##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##
##----------   Step 2 : Create an empty matrix containing        -----------------##
#-------------         all dates we want for the exercise        -----------------##
##--------------------------------------------------------------------------------##
##--------------------------------------------------------------------------------##


rawdata$date <- as.numeric(substr(rawdata$date,1,6))
rf$date<- as.numeric(substr(rf$date,1,6))

MonthsVector <- c("01","02","03","04","05","06","07","08","09","10","11","12")

startyear <- as.numeric(substr(196307,1,4))
startmonth <- substr(196307,5,6)
endyear <- as.numeric(substr(201012,1,4))
endmonth <- substr(201012,5,6)
nbofyears <- endyear - startyear
matrixofdates <- NULL
for(i in 0:nbofyears){
  for(r in 1:12){
    matrixofdates <- rbind(matrixofdates,c(startyear+i,MonthsVector[r]))
  }
}

matrixofdates <- matrixofdates[7:nrow(matrixofdates),]

newmatrix <- matrix(, nrow = nrow(matrixofdates), ncol = 2)

matrixofdates <- cbind(matrixofdates,newmatrix)

colnames(matrixofdates)[3:4] <- c("trim1","trim2")


##--------------------------------------------------------------------------------##
##----------  Step 3: create a loop to extract out-of-sample SDF  ----------------##
#-----------            in a rolling fashion                      ----------------##
##--------------------------------------------------------------------------------##

for(i in 1:nrow(matrixofdates)) {
  

    if (matrixofdates[i,2] %in% c("07") & i == 565){
      
      predictiondate<-as.numeric(paste(matrixofdates[i,1], matrixofdates[i,2],sep=""))
      predictionenddate <- as.numeric(paste(matrixofdates[i+5,1], matrixofdates[i+5,2],sep=""))  
      
      predictionyear<-as.numeric(substr(predictiondate,1,4))
      predictionmonth<-substr(predictiondate,5,6)
       lastmonth <- MonthsVector[as.numeric(predictionmonth)-1]
       lastyear<- predictionyear
       beginyear<- lastyear-30
       beginmonth<-predictionmonth
       begindate <- as.numeric(paste(beginyear, beginmonth,sep=""))
       lastdate <- as.numeric(paste(lastyear,lastmonth,sep=""))
       

       toy <- rawdata[which(rawdata$date >=begindate & rawdata$date <=predictionenddate),]
       toyrf <- rf[which(rf$date  >=begindate & rf$date <= predictionenddate),]
    

       totalx <- toy[,-1]
       toyrf <-toyrf[,2]

       totalrfm <- matrix(toyrf, nrow=length(toyrf),ncol=25)

       toyrfm <- totalrfm

       totalx  <- cbind(rep(1,nrow(totalx)),(totalx-toyrfm))
 
       x<- as.matrix(totalx[1:(nrow(totalx)-6),])
   
       ##apply trimming#
       supx <- apply(abs(x), 1, max)

       
       #first data uses q1 boundary#
       x1 <- x[(supx <= q1),]
       #second data uses q2 boundary#
       x2 <- x[(supx <= q2),]
       
       xpredict <- as.matrix(tail(totalx, 6))
       
       
      #optimization: PSN2#
       
       #objective function w/ trimming 2#
       objective <- function(par){

         x1.max <-pmax(x1 %*% par,rep(-1,nrow(x1)))
         ob1 <- (sum(x1.max*x1.max)/nrow(x1))/2 + sum(x1.max)/nrow(x1)
         ob2 <- sum(par*c(1,rep(0,ncol(x1)-1)))
         
  
         return(ob1-ob2)
       }
  
       #objective function w/ trimming 2#
       
       objective_1 <- function(par){
         x2.max <-pmax(x2 %*% par,rep(-1,nrow(x2)))
         ob1 <- (sum(x2.max*x2.max)/nrow(x2))/2 + sum(x2.max)/nrow(x2)
         ob2 <-  sum(par*c(1,rep(0,ncol(x2)-1)))
         return(ob1-ob2)
       }
       
       
       init <- rep(0,ncol(x))
       

       #first trimming boundary#    
       output_1 <-optim(init, objective)
       #second trimming boundary#  
       output_2 <-optim(init, objective_1)

       #estimated coefficient#
       est.coeff <- cbind(output_1$par,output_2$par)


       predict1 <- cbind((xpredict %*% est.coeff[,1]+rep(1,nrow(xpredict))),
                         (xpredict %*% est.coeff[,2]+rep(1,nrow(xpredict))))
       
       mhat <- predict1
  
       matrixofdates[i:(i+5),3:4] <- mhat
       
    } else if (matrixofdates[i,2] %in% c("07") & i!= 565){  
   
      predictiondate<-as.numeric(paste(matrixofdates[i,1], matrixofdates[i,2],sep=""))
      predictionenddate <- as.numeric(paste(matrixofdates[i+11,1], matrixofdates[i+11,2],sep=""))  
      
      predictionyear<-as.numeric(substr(predictiondate,1,4))
      predictionmonth<-substr(predictiondate,5,6)
      lastmonth <- MonthsVector[as.numeric(predictionmonth)-1]
      lastyear<- predictionyear
      beginyear<- lastyear-30
      beginmonth<-predictionmonth
      begindate <- as.numeric(paste(beginyear, beginmonth,sep=""))
      lastdate <- as.numeric(paste(lastyear,lastmonth,sep=""))
      
      
      toy <- rawdata[which(rawdata$date >=begindate & rawdata$date <=predictionenddate),]
      toyrf <- rf[which(rf$date  >=begindate & rf$date <= predictionenddate),]
      

      totalx <- toy[,-1]
      toyrf <-toyrf[,2]
      length(toyrf)
      totalrfm <- matrix(toyrf, nrow=length(toyrf),ncol=25)

      toyrfm <- totalrfm
      totalx  <- cbind(rep(1,nrow(totalx)),(totalx-toyrfm))
 
      x<- as.matrix(totalx[1:(nrow(totalx)-12),])
   
      
      ##apply trimming#
      supx <- apply(abs(x), 1, max)
     
      
      #first data uses q1 boundary#
      x1 <- x[(supx <= q1),]
      #second data uses q2 boundary#
      x2 <- x[(supx <= q2),]
      
      xpredict <- as.matrix(tail(totalx, 12))
      
      
      #objective function w/ trimming 1#
      objective <- function(par){
     
        x1.max <-pmax(x1 %*% par,rep(-1,nrow(x1)))
        ob1 <- (sum(x1.max*x1.max)/nrow(x1))/2 + sum(x1.max)/nrow(x1)
        ob2 <- sum(par*c(1,rep(0,ncol(x1)-1)))
        return(ob1-ob2)
      }
      
      #objective function w/ trimming 2#
      
      objective_1 <- function(par){
        x2.max <-pmax(x2 %*% par,rep(-1,nrow(x2)))
        ob1 <- (sum(x2.max*x2.max)/nrow(x2))/2 + sum(x2.max)/nrow(x2)
        ob2 <- sum(par*c(1,rep(0,ncol(x2)-1)))
        return(ob1-ob2)
      }
      
      
      init <- rep(0,ncol(x))
      
      
      #first trimming boundary#   
      output_1 <-optim(init, objective)
      #second trimming boundary# 
      output_2 <-optim(init, objective_1)
      
      #estimated coefficient#
      est.coeff <- cbind(output_1$par,output_2$par)
    

      predict1 <- cbind((xpredict %*% est.coeff[,1]+rep(1,nrow(xpredict))),
                        (xpredict %*% est.coeff[,2]+rep(1,nrow(xpredict))))
      
      mhat <- predict1

      matrixofdates[i:(i+11),3:4] <- mhat
    
  }
       
  print(paste(round(i/nrow(matrixofdates)*100),"% done for the loop.",sep=""))
  

}

matrixofdates[(matrixofdates[,3]<0),3] <- 0
matrixofdates[(matrixofdates[,4]<0),4] <- 0

##--------------------------------------------------------------------------------##
##----------  Step 3-1: save  out-of-sample SDF to a csv file      ---------------##
##--------------------------------------------------------------------------------##
write.csv(matrixofdates,file="25ff.sdf.est.PSN.2.trim.csv")


##--------------------------------------------------------------------------------##
##----------  Step 4: Conduct cross sectional regression tests     ---------------##
#-----------            using extracted SDF                       ----------------##
##--------------------------------------------------------------------------------##

pdata <- read.csv("25_Portfolios_5x5.csv", header=T)
pdata <- pdata[1:570,]

t <- (ncol(pdata)-1)

Table2.B.SDF.PSN.2.t <- matrix(NA,ncol = 5,nrow = 2)

colnames(Table2.B.SDF.PSN.2.t ) <-c("con","lamda","t.con","t.lamda", "r2")
rownames(Table2.B.SDF.PSN.2.t) <- c("trim1","trim2")

for (j in 3:4){
  
  
  #pick our recovered sdf
  sdf <- as.matrix(as.numeric(matrixofdates[,j]))
  
  #create an empty vector of length 25, which is the number of testing assets
  v.betas <- matrix(,  nrow = t, ncol = 1)
  
  #for each asset, run time series regresstion to get beta
  for (i in 1:t){
    
    y<- pdata[,(i+1)]
    fit1 <- lm( y ~ sdf)
    
    v.betas[i,1] <-coefficients(fit1)[2]
    
  }
  
  
  #calculate mean return for each asset
  pdata1 <- pdata[,2:ncol(pdata)]
  meanr <- as.vector(colMeans(pdata1))
  
  #second pass, run mean return on estimated beta
  finalfit <- lm(meanr ~ v.betas)
  output<-summary(finalfit)
  
  Table2.B.SDF.PSN.2.t[(j-2),1:2] <- output$coefficients[,1]
  Table2.B.SDF.PSN.2.t[(j-2),3:4] <- output$coefficients[,3]
  Table2.B.SDF.PSN.2.t[(j-2),5] <- output$adj.r.squared
  
}



write.csv(Table2.B.SDF.PSN.2.t,file="Table2.B.SDF.PSN.2.trim.csv")


